/**
 * Types related to the ClinicalTrials.gov XML file. This is based on the ClinicalTrials.gov XML schema:
 * https://clinicaltrials.gov/ct2/html/images/info/public.xsd
 *
 * This is a *partial* set of types. It's based on the object that would be created via xml2js, assuming that the
 * given XML file is valid.
 */
/**
 * For documentation purposes, indicates an element that will only ever appear once in a valid document. It's
 * implmented as a TypeScript tuple with a single element, which matches how xml2js will ultimately parse the XML.
 */
declare type One<T> = [T];
/**
 * For documentation purposes, indicates an element that can appear any number of times. (Conceptually, an empty array
 * isn't actually valid for these types.)
 */
declare type Unbounded<T> = Array<T>;
/**
 * For documentation purposes, a field that is an XML Schema Integer.
 * (Note that technically, integer is unbounded - that is, may be any integer of any length.)
 */
export declare type XSInteger = string;
/**
 * For documentation purposes, a field that is an XML Schema positiveInteger.
 */
export declare type XSPositiveInteger = string;
export declare type ActualEnum = 'Actual' | 'Anticipated' | 'Estimate';
export declare type YesNoEnum = 'Yes' | 'No';
/**
 * Actually needs to match a pattern, but TypeScript doesn't support that. From
 * the schema:
 *
 * Dates may now include a day
 *     Month Day, Year    Or    Month Year
 * Necessary to accurately calculate deadlines
 *
 * The pattern is: (Unknown|((January|February|March|April|May|June|July|August|September|October|November|December) (([12]?[0-9]|30|31)\, )?[12][0-9]
 */
export declare type VariableDate = string;
export declare type VariableDateStruct = {
    $: {
        type?: ActualEnum;
    };
    _: VariableDate;
} | VariableDate;
export interface RequiredHeaderStruct {
    download_date: One<string>;
    link_text: One<string>;
    url: One<string>;
}
export interface IdInfoStruct {
    org_study_id?: One<string>;
    secondary_id?: Unbounded<string>;
    nct_id: One<string>;
    nct_alias?: Unbounded<string>;
}
export declare type AgencyClassEnum = 'NIH' | 'U.S. Fed' | 'Industry' | 'Other';
export interface SponsorStruct {
    agency: One<string>;
    agency_class?: One<AgencyClassEnum>;
}
export interface SponsorsStruct {
    lead_sponsor: One<SponsorStruct>;
    collaborator?: Unbounded<SponsorStruct>;
}
export interface OversightInfoStruct {
    has_dmc?: One<YesNoEnum>;
    is_fda_regulated_drug?: One<YesNoEnum>;
    is_fda_regulated_device?: One<YesNoEnum>;
    is_unapproved_device?: One<YesNoEnum>;
    is_ppsd?: One<YesNoEnum>;
    is_us_export?: One<YesNoEnum>;
}
export interface ExpandedAccessInfoStruct {
    expanded_access_type_individual?: One<YesNoEnum>;
    expanded_access_type_intermediate?: One<YesNoEnum>;
    expanded_access_type_treatment?: One<YesNoEnum>;
}
export interface StudyDesignInfoStruct {
    allocation?: One<string>;
    intervention_model?: One<string>;
    intervention_model_description?: One<string>;
    primary_purpose?: One<string>;
    observational_model?: One<string>;
    time_perspective?: One<string>;
    masking?: One<string>;
    masking_description?: One<string>;
}
export interface protocol_outcome_struct {
    measure: One<string>;
    time_frame?: One<string>;
    description?: One<string>;
}
export declare type EnrollmentStruct = {
    $: {
        type: ActualEnum;
    };
    _: XSInteger;
} | XSInteger;
export interface ArmGroupStruct {
    arm_group_label: One<string>;
    arm_group_type?: One<string>;
    description?: One<string>;
}
export declare type InterventionTypeEnum = 'Behavioral' | 'Biological' | 'Combination Product' | 'Device' | 'Diagnostic Test' | 'Dietary Supplement' | 'Drug' | 'Genetic' | 'Procedure' | 'Radiation' | 'Other';
export interface InterventionStruct {
    intervention_type: One<InterventionTypeEnum>;
    intervention_name: One<string>;
    description?: One<string>;
    arm_group_label?: Unbounded<string>;
    other_name?: Unbounded<string>;
}
export interface TextblockStruct {
    textblock: One<string>;
}
export declare type SamplingMethodEnum = 'Probability Sample' | 'Non-Probability Sample';
export declare type GenderEnum = 'Female' | 'Male' | 'All';
/**
 * This is another XML schema pattern. The pattern is:
 * N/A|([1-9][0-9]* (Year|Years|Month|Months|Week|Weeks|Day|Days|Hour|Hours|Minute|Minutes))
 */
export declare type AgePattern = string;
export interface EligibilityStruct {
    study_pop?: One<TextblockStruct>;
    sampling_method?: One<SamplingMethodEnum>;
    criteria?: One<TextblockStruct>;
    gender: One<GenderEnum>;
    gender_based?: One<YesNoEnum>;
    gender_description?: One<string>;
    minimum_age: One<AgePattern>;
    maximum_age: One<AgePattern>;
    healthy_volunteers?: One<string>;
}
export interface ContactStruct {
    first_name?: One<string>;
    middle_name?: One<string>;
    last_name?: One<string>;
    degrees?: One<string>;
    phone?: One<string>;
    phone_ext?: One<string>;
    email?: One<string>;
}
export declare type RoleEnum = 'Principal Investigator' | 'Sub-Investigator' | 'Study Chair' | 'Study Director';
export interface InvestigatorStruct {
    first_name?: One<string>;
    middle_name?: One<string>;
    last_name: One<string>;
    degrees?: One<string>;
    role?: One<RoleEnum>;
    affiliation?: One<string>;
}
export interface AddressStruct {
    city: One<string>;
    state?: One<string>;
    zip?: One<string>;
    country: One<string>;
}
export interface FacilityStruct {
    name?: One<string>;
    address?: One<AddressStruct>;
}
export declare type RecruitmentStatusEnum = 'Active, not recruiting' | 'Completed' | 'Enrolling by invitation' | 'Not yet recruiting' | 'Recruiting' | 'Suspended' | 'Terminated' | 'Withdrawn';
export declare type ExpandedAccessStatusEnum = 'Available' | 'No longer available' | 'Temporarily not available' | 'Approved for marketing';
export declare type RedactedRecordStatusEnum = 'Withheld';
export declare type UnknownStatusEnum = 'Unknown status';
/**
 * Valid values for overall status depend on the value of the study type:
 *  - For studies that have not been updated in 2 years, the value unknown_status_enum
 *  - For Interventional or Observational studies, values from recruitment_status_enum
 *  - For Expanded Access studies, values from expanded_access_status_enum
 *  - Otherwise, values from redacted_record_status_enum
 *
 * This enum is also the valid values for location status, but the rules for which
 * locations are included or witheld in the public xml are subject to change.
 * You may not see all possible values.
 */
export declare type StatusEnum = RecruitmentStatusEnum | ExpandedAccessStatusEnum | RedactedRecordStatusEnum | UnknownStatusEnum;
export interface LocationStruct {
    facility?: One<FacilityStruct>;
    status?: One<StatusEnum>;
    contact?: One<ContactStruct>;
    contact_backup?: One<ContactStruct>;
    investigator?: Unbounded<InvestigatorStruct>;
}
export interface CountriesStruct {
    country?: Unbounded<string>;
}
export interface LinkStruct {
    url: One<string>;
    description?: One<string>;
}
export interface ReferenceStruct {
    citation?: One<string>;
    PMID?: One<XSPositiveInteger>;
}
export declare type ResponsiblePartyTypeEnum = 'Sponsor' | 'Principal Investigator' | 'Sponsor-Investigator';
/**
 * old style
 */
export interface ResponsiblePartyStructOldStyle {
    name_title?: One<string>;
    organization?: One<string>;
}
/**
 * new style, used in newer records
 */
export interface ResponsiblePartyStructNewStyle {
    responsible_party_type: One<ResponsiblePartyTypeEnum>;
    investigator_affiliation?: One<string>;
    investigator_full_name?: One<string>;
    investigator_title?: One<string>;
}
export declare type ResponsiblePartyStruct = ResponsiblePartyStructOldStyle | ResponsiblePartyStructNewStyle;
export interface BrowseStruct {
    mesh_term?: Unbounded<string>;
}
export interface PatientDataStruct {
    sharing_ipd: One<string>;
    ipd_description?: One<string>;
    ipd_info_type?: Unbounded<string>;
    ipd_time_frame?: One<string>;
    ipd_access_criteria?: One<string>;
    ipd_url?: One<string>;
}
export interface StudyDocStruct {
    doc_id?: One<string>;
    doc_type?: One<string>;
    doc_url?: One<string>;
    doc_comment?: One<string>;
}
export interface StudyDocsStruct {
    study_doc: Unbounded<StudyDocStruct>;
}
export interface provided_document_struct {
    document_type?: One<string>;
    document_has_protocol?: One<string>;
    document_has_icf: One<string>;
    document_has_sap?: One<string>;
    document_date?: One<string>;
    document_url?: One<string>;
}
export interface provided_document_section_struct {
    provided_document: provided_document_struct;
}
export interface PendingResultsStruct {
    submitted?: VariableDateStruct[];
    returned?: VariableDateStruct[];
    submission_canceled?: VariableDateStruct[];
}
export interface GroupStruct {
    $: {
        group_id: string;
    };
    title?: One<string>;
    description?: One<string>;
}
export interface ParticipantsStruct {
    $: {
        group_id: string;
        count?: string;
    };
    _: string;
}
export interface MilestoneStruct {
    title: One<string>;
    participants_list: One<{
        participants: ParticipantsStruct[];
    }>;
}
export declare type StudyTypeEnum = 'Expanded Access' | 'Interventional' | 'N/A' | 'Observational' | 'Observational [Patient Registry]';
export declare type PhaseEnum = 'N/A' | 'Early Phase 1' | 'Phase 1' | 'Phase 1/Phase 2' | 'Phase 2' | 'Phase 2/Phase 3' | 'Phase 3' | 'Phase 4';
export declare type BiospecRetentionEnum = 'None Retained' | 'Samples With DNA' | 'Samples Without DNA';
export interface ClinicalStudy {
    $?: {
        rank?: string;
    };
    required_header: One<RequiredHeaderStruct>;
    id_info: One<IdInfoStruct>;
    brief_title: One<string>;
    acronym?: One<string>;
    official_title?: One<string>;
    sponsors: One<SponsorsStruct>;
    source: One<string>;
    oversight_info?: One<OversightInfoStruct>;
    brief_summary?: One<TextblockStruct>;
    detailed_description?: One<TextblockStruct>;
    overall_status: One<StatusEnum>;
    last_known_status?: One<StatusEnum>;
    why_stopped?: One<string>;
    start_date?: One<VariableDateStruct>;
    completion_date?: One<VariableDateStruct>;
    primary_completion_date?: One<VariableDateStruct>;
    phase?: One<PhaseEnum>;
    study_type: One<StudyTypeEnum>;
    has_expanded_access?: One<YesNoEnum>;
    expanded_access_info?: One<ExpandedAccessInfoStruct>;
    study_design_info?: One<StudyDesignInfoStruct>;
    target_duration?: One<string>;
    number_of_arms?: One<XSInteger>;
    number_of_groups?: One<XSInteger>;
    enrollment?: One<EnrollmentStruct>;
    condition?: Unbounded<string>;
    arm_group?: Unbounded<ArmGroupStruct>;
    intervention?: Unbounded<InterventionStruct>;
    eligibility?: One<EligibilityStruct>;
    overall_contact?: One<ContactStruct>;
    overall_contact_backup?: One<ContactStruct>;
    location?: Unbounded<LocationStruct>;
}
export declare function isClinicalStudy(o: unknown): o is ClinicalStudy;
export {};
