/**
 * This file contains a backup system for finding necessary trial information if
 * your matching service does not provide it.
 *
 * The intended usages is essentially:
 *
 * Create the service:
 *
 * const ctgService = await createClinicalTrialsGovService('temp-data');
 *
 * Then, when ResearchStudy objects exist that need to be filled out, assuming
 * that studies is a ResearchStudy[]:
 *
 * await ctgService.updateResearchStudies(studies);
 *
 * This will fill out whatever can be filled out within the given studies.
 */
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import fs from 'fs';
import * as http from 'http';
import * as stream from 'stream';
import { ResearchStudy } from 'fhir/r4';
import { ClinicalStudy, StatusEnum } from './clinicalstudy';
/**
 * Logger type from the NodeJS utilities. (The TypeScript definitions for Node
 * don't bother naming this type.)
 */
declare type Logger = (message: string, ...param: unknown[]) => void;
/**
 * More for documentation purposes than anything else, this marks strings that are expected to be a valid NCT number.
 * (Generally they aren't checked to ensure they're valid, the methods involved will return nothing if they are not.)
 */
export declare type NCTNumber = string;
/**
 * This is the actual type generated by xml2js when the XML file is parsed.
 */
interface TrialBackup {
    clinical_study: ClinicalStudy;
}
export declare function isTrialBackup(o: unknown): o is TrialBackup;
/**
 * The URL for the "system" field of identifiers that indicate the
 * ClinicalTrial.gov identifier, or NTC number, of a clinical trial.
 */
export declare const CLINICAL_TRIAL_IDENTIFIER_CODING_SYSTEM_URL = "http://clinicaltrials.gov/";
/**
 * Checks to ensure that a given NCT number is valid. This checks to make sure
 * it matches the regular expression /^NCT[0-9]{8}$/.
 * @param nctNumber the NCT number to check
 */
export declare function isValidNCTNumber(nctNumber: string): boolean;
/**
 * Finds the NCT number specified for the given ResearchStudy, assuming there is
 * one. This requires an identifier on the given ResearchStudy that either
 * belongs to the coding system "http://clinicaltrials.gov/" or is a valid NCT
 * number as determined by #isValidNCTNumber.
 *
 * This will return the value of the first identifier belonging to the coding
 * system "http://clinicaltrials.gov/", and if there is no identifier, return
 * the first identifier value that is considered valid.
 *
 * @param study the research study
 * @returns the NCT number or null if none
 */
export declare function findNCTNumber(study: ResearchStudy): NCTNumber | null;
/**
 * Finds all the NCT numbers within the given list of studies. Returns a map of
 * NCT numbers to the research studies they match. If multiple research studies
 * have the same NCT ID, then an array will be used to contain all matching
 * studies. (This should be very uncommon but is supported anyway.)
 * @param studies the NCT numbers found, if any
 */
export declare function findNCTNumbers(studies: ResearchStudy[]): Map<NCTNumber, ResearchStudy | Array<ResearchStudy>>;
/**
 * Attempts to parse the given string as XML and into a ClinicalStudy object. If the XML parsing succeeds but the XML
 * is not recognized as a valid ClinicalStudy, this returns null. If the XML parsing fails, the Promise will be rejected.
 * @param fileContents the contents of the file as a string
 * @param log an optional logger
 * @returns a promise that resolves to the parsed object or null
 */
export declare function parseClinicalTrialXML(fileContents: string, log?: Logger): Promise<ClinicalStudy | null>;
export declare function convertClincalStudyStatusToFHIRStatus(status: StatusEnum): ResearchStudy['status'] | undefined;
/**
 * Subset of the Node.js fs module necessary to handle the cache file system, allowing it to be overridden if
 * necessary.
 */
export interface FileSystem {
    readFile: (path: fs.PathOrFileDescriptor, options: {
        encoding: BufferEncoding;
        flag?: string;
    }, callback: (err: NodeJS.ErrnoException | null, data: string) => void) => void;
    mkdir: (path: string, callback: (err: NodeJS.ErrnoException | null) => void) => void;
    readdir: (path: string, callback: (err: NodeJS.ErrnoException | null, files: string[]) => void) => void;
    stat: (path: string, callback: (err: NodeJS.ErrnoException | null, stat: fs.Stats) => void) => void;
    createWriteStream: (path: string) => NodeJS.WritableStream;
    unlink: (path: string, callback: (err: NodeJS.ErrnoException | null) => void) => void;
}
/**
 * A cache entry. Cache entries basically operate in two modes: an entry that is pending being written based on a ZIP
 * file, and a file that has a backing file ready to be loaded.
 *
 * The pending state is kind of weird because an entry is pending once a download has been requested for it, but cannot
 * be properly fulfilled until the file data has been saved. There's no really good way to wrap that in a Promise so
 * instead the resolve method is stored until the pending state can be resolved.
 */
export declare class CacheEntry {
    filename: string;
    private _cache;
    private _createdAt;
    private _lastAccess;
    /**
     * The pending status. Either a Promise (in which case it is not only pending, but has things actively waiting for
     * it), or true, in which case it is pending but the promise is yet to be created, or false if pending but the promise
     * has not been created. This is so that no Promise is created if nothing ever needs it, which avoids cases where the
     * Promise won't have a catch handler attached to it.
     */
    private _pending;
    /**
     * Create a new cache entry.
     * @param filename the file name of the entry
     * @param options the file stats (if the file exists) or a flag indicating it's pending (being downloaded still)
     */
    constructor(cache: ClinicalTrialsGovService, filename: string, options: {
        stats?: fs.Stats;
        pending?: boolean;
    });
    /**
     * Gets the time when this entry was initially created. If null, that indicates that the entry was never successfully
     * created - it's still waiting for data to be downloaded.
     */
    get createdAt(): Date | null;
    /**
     * Gets the last time the cache entry was accessed. This is updated whenever
     * #load() is called.
     */
    get lastAccess(): Date;
    /**
     * Determine if the cache entry is still pending: data for it has not yet been saved.
     */
    get pending(): boolean;
    /**
     * Check if the last access time is before the given date
     * @param date the date to check
     * @returns true if the last access time was before the given date
     */
    lastAccessedBefore(date: Date): boolean;
    /**
     * Indicates that the entry has been located somewhere and that data for it is now being prepared.
     */
    found(): void;
    /**
     * Resolves the pending state (if the entry was pending), otherwise does nothing. This does not change the "found"
     * status - if ready() is called without found(), createdAt remains null and the entry may be removed as pointing to
     * a record that does not exist.
     */
    ready(): void;
    /**
     * Forcibly fail the entry.
     * @param e the error to fail the entry with
     */
    fail(e: Error | string): void;
    /**
     * Loads the underlying file. If the entry is still pending, then the file is read once the entry is ready. This may
     * resolve to null if the clinical study does not exist in the results.
     */
    load(): Promise<ClinicalStudy | null>;
    /**
     * Always attempt to read the file, regardless of whether or not the entry is pending.
     */
    readFile(): Promise<ClinicalStudy | null>;
    /**
     * Remove the entry from the cache. All this does is delete the underlying file. The returned Promise is more for
     * error handling than anything else.
     */
    remove(): Promise<void>;
}
/**
 * Create a directory if it does not already exist. If it does exist (or some other file exists in its place), this
 * still resolves rather than raising an error. This used to check if the paths were really directories, but since the
 * final step is now reading the file entries from the directory anyway, that check ensures everything is a directory
 * anyway.
 *
 * @param path the path to create (will be created recursively)
 * @return a Promise that resolves as true if the directory was newly created or false if it existed
 */
export declare function mkdir(fs: FileSystem, path: string): Promise<boolean>;
export interface ClinicalTrialsGovServiceOptions {
    /**
     * The logging function to use to log debug information. If not set the default is `debuglog('ctgovservice')` from
     * the `util` module in Node.js.
     */
    log?: Logger;
    /**
     * Time in milliseconds to expire files after.
     */
    expireAfter?: number;
    /**
     * Interval in milliseconds to run periodic cache clean-ups. If set to 0 or infinity, never run cleanups.
     */
    cleanInterval?: number;
    /**
     * If given, rather than use the default Node.js file system, use the given override. Mainly used for tests but may
     * also be useful in instances where a "real" file system is unavailable.
     */
    fs?: FileSystem;
}
/**
 * System to fill in data on research studies based on the trial data from
 * https://clinicaltrials.gov/
 *
 * The current system works as follows:
 *
 * Call updateResearchStudies() with the ResearchStudy objects to update. It will locate all ResearchStudy objects that
 * contain an NCT ID. It will then:
 *
 * 1. Call downloadTrials() to download the ClinicalStudy data from clinicaltrials.gov. It starts an HTTPS request to
 *    download the trial results and, if successful, passes the response stream off to extractResults().
 * 2. extractResults() saves the stream its given to a temporary ZIP file and extracts that file to a temporary
 *    directory. Once the ZIP is extracted, it deletes the temporary ZIP. It then returns the directory where the files
 *    were extracted.
 * 3. With the path to the extracted ZIP, updateResearchStudies() can now use the downloaded trial data to update the
 *    trials.
 * 4. Before resolving, the temporary directory is deleted.
 *
 * A future version will likely change this to make it so that rather than immediately deleting the temporary trial
 * data, it's kept around in a least-recently-used cache and only deleted as the cache exceeds a given size. This will
 * add a few new public APIs to enable the retrieval of clinical study data by NCT ID. Right now, though, there is no
 * way to extract the clinical study data directly.
 */
export declare class ClinicalTrialsGovService {
    readonly dataDir: string;
    /**
     * Internal value to track temporary file names.
     */
    private tempId;
    /**
     * Log used to log debug information. Either passed in at creation time or created via the Node.js util.debuglog
     * function. With the latter, activate by setting the NODE_DEBUG environment variable to "ctgovservice" (or include it
     * in a comma separated list of debugging modules to include).
     */
    readonly log: Logger;
    private _maxTrialsPerRequest;
    /**
     * The maximum number of trials to attempt to download in a single request. Each NCT ID increases the URI size by
     * effectively 15 bytes: essentially each NCT ID becomes '+OR+NCT12345678'. The request path "counts" as a header and
     * has to be smaller than maxHeaderSize given to the client. By default, this uses the default client, so the client
     * should be the default HTTPS client. In theory this would support around 500 per request, instead this defaults to
     * 128 which is a nice round number. (In binary, at least.)
     */
    get maxTrialsPerRequest(): number;
    set maxTrialsPerRequest(newValue: number);
    private _expirationTimeout;
    /**
     * The expiration timeout, the time in milliseconds after a cache entry should be considered expired. The default
     * is 1 hour.
     */
    get expirationTimeout(): number;
    set expirationTimeout(value: number);
    private _cleanupIntervalMillis;
    /**
     * The interval between cleanup sweeps. Set to 0 (or any negative value) or Infinity to disable periodic cleanup. Note
     * that the value is limited to the range 60000 (a minute)-2147483647 (the maximum value of a signed 32-bit integer)
     * as that maximum value is the maximum allowed timeout for a timer within Node.js. If the value is set to a negative
     * number or Infinity, the retrieved value will be 0, to indicate that the interval will not be set.
     */
    get cleanupInterval(): number;
    set cleanupInterval(value: number);
    /**
     * The maximum allowed entry size before it is rejected. Default is 128MB, which is hopefully well more than necessary
     * for any reasonable clinical trial XML.
     */
    maxAllowedEntrySize: number;
    private cacheDataDir;
    /**
     * Actual cache of NCT IDs to their cached values. This is the "real cache" versus whatever's on the filesystem.
     * (The NCT ID is in the form that returns true from isValidNCTNumber, so NCTnnnnnnnn where n are digits.)
     */
    private cache;
    private cleanupTimeout;
    /**
     * The filesystem being used by the cache.
     */
    readonly fs: FileSystem;
    /**
     * Creates a new instance. This will not initialize the cache or load anything, this merely creates the object. Use
     * the #init() method to initialize the service and load existing data. (Or use createClinicalTrialsGovService() do
     * construct and initialize at the same time.)
     *
     * @param dataDir the directory to use for cache data
     * @param log an optional function to receive debug log messages. By default this uses util.debuglog('ctgovservice')
     *     meaning that the log can be activated by setting NODE_DEBUG to "ctgovservice"
     */
    constructor(dataDir: string, options?: ClinicalTrialsGovServiceOptions);
    /**
     * Creates the necessary directories if they do not exist, and loads any existing data into the cache. Parent
     * directories of the cache will not be created automatically, they must already exist.
     */
    init(): Promise<void>;
    /**
     * Sets the timeout interval assuming it was set.
     */
    private setCleanupTimeout;
    /**
     * Shuts down the service, doing any final necessary cleanup. At present this stops any running timers. The Promise
     * returned currently resolves immediately, but in the future, it may resolve asynchronously when the service has been
     * cleanly shut down.
     */
    destroy(): Promise<void>;
    /**
     * This attempts to load the cache from the filesystem.
     */
    private restoreCacheFromFS;
    private createCacheEntry;
    /**
     * Attempts to update the given research studies with data from within this store. This returns a Promise that
     * resolves when the research studies have been updated with any available additional data. The Promise may be
     * rejected if the process fails entirely (that is, clinicaltrials.gov could not be contacted at all, or an I/O error
     * occurs).
     *
     * @param studies the studies to attempt to update
     * @returns a Promise that resolves when the studies are updated. It will resolve with the same array that was passed
     *     in - this updates the given objects, it does not clone them and create new ones.
     */
    updateResearchStudies(studies: ResearchStudy[]): Promise<ResearchStudy[]>;
    /**
     * Tells the cache to delete all expired cached files. The entries are removed from access immediately, but a Promise
     * is returned that resolves when all the underlying data is cleaned up. This Promise is more for error handling than
     * anything else.
     */
    removeExpiredCacheEntries(): Promise<void>;
    /**
     * Ensures that a given set of NCT IDs is available within the service, assuming they exist on ClinicalTrials.gov.
     * @param ids the IDs to ensure are available
     * @returns a Promise that resolves once any downloads have completed
     */
    ensureTrialsAvailable(ids: string[]): Promise<void>;
    ensureTrialsAvailable(studies: ResearchStudy[]): Promise<void>;
    /**
     * Downloads the given trials from ClinicalTrials.gov and stores them in the data directory. Note that this will
     * always replace trials if they exist.
     *
     * @param ids the IDs of the trials to download
     * @returns a Promise that resolves to the path where the given IDs were downloaded
     */
    protected downloadTrials(ids: string[]): Promise<void>;
    /**
     * Provides a method that can be overridden to alter how a web request is
     * made. Defaults to simply calling https.get directly.
     * @param url the URL to get
     * @param callback the callback
     */
    protected getURL(url: string, callback: (res: http.IncomingMessage) => void): http.ClientRequest;
    /**
     * Internal method to create a temporary file within the data directory. Temporary files created via this method are
     * not automatically deleted and need to be cleaned up by the caller.
     */
    private createTemporaryFileName;
    /**
     * Create a path to the data file that stores data about a cache entry.
     * @param nctNumber the NCT number
     */
    private pathForNctNumber;
    /**
     * Extract a given set of results, updating any cache results with the contents.
     * @param results a readable stream that contains a ZIP file that contains the results
     * @returns a Promise that resolves when the files have been extracted
     */
    protected extractResults(results: stream.Readable): Promise<void>;
    /**
     * Extracts clinical trials within a ZIP file.
     * @param zipPath the path to the ZIP file
     * @param opts the options
     */
    private extractZip;
    private addCacheEntry;
    /**
     * Loads a ClinicalStudy from an extracted dataset. This will never download a copy, this will only ever return from
     * within the cache.
     * @param nctNumber the NCT number
     * @returns a Promise that resolves to either the parsed ClinicalStudy or to null if the ClinicalStudy does not exist
     */
    getCachedClinicalStudy(nctNumber: NCTNumber): Promise<ClinicalStudy | null>;
    /**
     * The provides a stub that handles updating the research study with data from a clinical study downloaded from the
     * ClinicalTrials.gov website. This primarily exists as a stub to allow the exact process which updates a research
     * study to be overridden if necessary.
     *
     * @param researchStudy the research study to update
     * @param clinicalStudy the clinical study to update it with
     */
    updateResearchStudy(researchStudy: ResearchStudy, clinicalStudy: ClinicalStudy): void;
    /**
     * Creates and initializes a new service for retrieving data from http://clinicaltrials.gov/. This will automatically
     * invoke the init method to create the directory if it doesn't exist and load any existing data if it does. Note that
     * the init method will not create missing parent directories - the path to the cache directory must already exist
     * minus the cache directory itself. If the cache directory cannot be created the Promise will be rejected with the
     * error preventing it from being created.
     *
     * @param dataDir the data directory
     * @param options additional options that can be set to further configure the trial service
     * @returns a Promise that resolves when the service is ready
     */
    static create(dataDir: string, options?: ClinicalTrialsGovServiceOptions): Promise<ClinicalTrialsGovService>;
}
/**
 * Creates and initializes a new service for retrieving data from http://clinicaltrials.gov/. This is the same as
 * ClinicalTrialsGovService.create, see that method for details.
 *
 * @param dataDir the data directory
 * @param options additional options that can be set to further configure the trial service
 * @returns a Promise that resolves when the service is ready
 */
export declare function createClinicalTrialsGovService(dataDir: string, options?: ClinicalTrialsGovServiceOptions): Promise<ClinicalTrialsGovService>;
/**
 * Updates a research study with data from a clinical study off the ClinicalTrials.gov website. This will only update
 * fields that do not have data, it will not overwrite any existing data.
 *
 * Mapping as defined by https://www.hl7.org/fhir/researchstudy-mappings.html#clinicaltrials-gov
 *
 * @param result the research study to update
 * @param study the clinical study to use to update (this takes a partial as the ClinicalStudy type describes the XML
 * as the schema defines it, so this is designed to handle invalid XML that's missing information that should be
 * required)
 */
export declare function updateResearchStudyWithClinicalStudy(result: ResearchStudy, study: Partial<ClinicalStudy>): ResearchStudy;
export {};
