import { Coding } from 'fhir/r4';
/**
 * Enumeration of the possible code systems.
 * Source: https://masteringjs.io/tutorials/fundamentals/enum
 */
export declare class CodeSystemEnum {
    static ICD10: CodeSystemEnum;
    static SNOMED: CodeSystemEnum;
    static RXNORM: CodeSystemEnum;
    static AJCC: CodeSystemEnum;
    static LOINC: CodeSystemEnum;
    static NIH: CodeSystemEnum;
    static HGNC: CodeSystemEnum;
    static HL7: CodeSystemEnum;
    system: string;
    /**
     * Constructor
     * @param system The system name.
     */
    constructor(system: string);
    toString(): string;
}
/**
 * A class that acts a Code Mapper that maps an input JSON's codes to their associated profiles.
 */
export declare class CodeMapper {
    code_map: Map<string, string[]>;
    /**
     * Constructor for a Code Mapper.
     * @param code_mapping_file The file that dictates the code mapping.
     */
    constructor(code_mapping_file: {
        [key: string]: ProfileSystemCodes;
    });
    /**
     * Converts the JSON object to the Map<Code -> Profile[]> format.
     * @param obj The JSON object.
     * @returns Map<String. Map<String -> List<String>>>
     */
    static convertJsonToCodeMap(obj: {
        [key: string]: ProfileSystemCodes;
    }): Map<string, string[]>;
    /**
     * Extracts the code mappings for the given list of codings.
     * @param codings The codings to map to profiles.
     * @returns The list of mapped strings for the codings.
     */
    extractCodeMappings(codings: Coding[]): string[];
    /**
     * Returns whether the given coding equals the given code attributes.
     * @param input_coding The coding to check against.
     * @param system The system of the code to compare to.
     * @param code The code of the code to compare to.
     */
    static codesEqual(input_coding: Coding, system: CodeSystemEnum, code: string): boolean;
    /**
     * Normalize the code system to a consistent format.
     * @param codeSystem  The code system to normalize.
     * @returns The normalized code system.
     */
    static normalizeCodeSystem(codeSystem: string): CodeSystemEnum;
}
/**
 * Describes the format that the JSON object should be made up of.
 */
interface ProfileSystemCodes {
    [system: string]: string[];
}
export {};
