import * as fhir from 'fhir/r4';
export declare type FHIRPath = string;
export interface ReasonReference {
    reference?: string;
    display?: string;
    meta_profile?: string;
}
export interface Quantity {
    value?: number | string;
    comparator?: string;
    unit?: string;
    system?: string;
    code?: string;
}
export interface Ratio {
    numerator?: Quantity;
    denominator?: Quantity;
}
export interface BaseFhirResource {
    coding: fhir.Coding[];
}
export interface CancerConditionParent extends BaseFhirResource {
    clinicalStatus: fhir.Coding[];
    meta_profile: string;
    id: string;
}
export interface PrimaryCancerCondition extends CancerConditionParent {
    histologyMorphologyBehavior: fhir.Coding[];
}
export interface SecondaryCancerCondition extends CancerConditionParent {
    bodySite: fhir.Coding[];
}
export interface CancerRelatedProcedureParent extends BaseFhirResource {
    bodySite: fhir.Coding[];
}
export interface CancerRelatedRadiationProcedure extends CancerRelatedProcedureParent {
    mcodeTreatmentIntent: fhir.Coding[];
}
export interface CancerRelatedSurgicalProcedure extends CancerRelatedProcedureParent {
    reasonReference: ReasonReference;
}
export interface TumorMarker extends BaseFhirResource {
    valueQuantity: Quantity[];
    valueRatio: Ratio[];
    valueCodeableConcept: fhir.Coding[];
    interpretation: fhir.Coding[];
}
export interface CancerGeneticVariant extends BaseFhirResource {
    component: CancerGeneticVariantComponent;
    valueCodeableConcept: fhir.Coding[];
    interpretation: fhir.Coding[];
}
export interface CancerGeneticVariantComponent {
    geneStudied: CancerGeneticVariantComponentType[];
    genomicsSourceClass: CancerGeneticVariantComponentType[];
}
export interface CancerGeneticVariantComponentType {
    code: {
        coding: fhir.Coding[];
    };
    valueCodeableConcept: {
        coding: fhir.Coding[];
    };
    interpretation: {
        coding: fhir.Coding[];
    };
}
/**
 * Class that extracts mCODE data from a patient record.
 */
export declare class mCODEextractor {
    /**
     * mCODE Resources.
     */
    private primaryCancerCondition;
    private secondaryCancerCondition;
    private TNMClinicalStageGroup;
    private TNMPathologicalStageGroup;
    private birthDate;
    private tumorMarker;
    private cancerGeneticVariant;
    private cancerRelatedRadiationProcedure;
    private cancerRelatedSurgicalProcedure;
    private cancerRelatedMedicationStatement;
    private ecogPerformanceStatus?;
    private karnofskyPerformanceStatus?;
    /**
     * Getters.
     */
    getPrimaryCancerConditions(): PrimaryCancerCondition[];
    getSecondaryCancerConditions(): SecondaryCancerCondition[];
    getTNMclinicalStageGroup(): fhir.Coding[];
    getTNMpathologicalStageGroup(): fhir.Coding[];
    getBirthDate(): string;
    getTumorMarkers(): TumorMarker[];
    getCancerGeneticVariants(): CancerGeneticVariant[];
    getCancerRelatedRadiationProcedures(): CancerRelatedRadiationProcedure[];
    getCancerRelatedSurgicalProcedures(): CancerRelatedSurgicalProcedure[];
    getCancerRelatedMedicationStatements(): fhir.Coding[];
    getEcogPerformanceStatus(): number;
    getKarnofskyPerformanceStatus(): number;
    /**
     * Constructor.
     * @param patientBundle The patient bundle to build the mCODE mapping from.
     */
    constructor(patientBundle: fhir.Bundle);
    /**
     * Resource lookup.
     * @param resource
     * @param path
     * @param environment
     * @returns
     */
    private lookup;
    /**
     * Checks if the given profile constains a match to the given key.
     */
    private resourceProfile;
    /**
     * Checks whether the given list contains the given coding.
     * @param codingList
     * @param coding
     * @returns
     */
    private contains;
    /**
     * Adds the given coding to the given coding list.
     * @param codingList
     * @param codes
     * @returns
     */
    private addCoding;
    /**
     * Returns whether the given list contains the given procedure.
     * @param procedureList
     * @param procedure
     * @returns
     */
    listContainsProcedure(procedureList: CancerRelatedProcedureParent[], procedure: CancerRelatedProcedureParent): boolean;
}
