"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mCODEextractor = void 0;
const fhirpath_1 = __importDefault(require("fhirpath"));
/**
 * Class that extracts mCODE data from a patient record.
 */
class mCODEextractor {
    /**
     * Constructor.
     * @param patientBundle The patient bundle to build the mCODE mapping from.
     */
    constructor(patientBundle) {
        this.primaryCancerCondition = [];
        this.TNMClinicalStageGroup = [];
        this.TNMPathologicalStageGroup = [];
        this.secondaryCancerCondition = [];
        this.tumorMarker = [];
        this.cancerRelatedRadiationProcedure = [];
        this.cancerRelatedSurgicalProcedure = [];
        this.cancerRelatedMedicationStatement = [];
        this.cancerGeneticVariant = [];
        this.birthDate = 'N/A';
        if (patientBundle != undefined && patientBundle.entry) {
            for (const entry of patientBundle.entry) {
                if (!('resource' in entry) || (!entry.resource)) {
                    // Skip bad entries
                    continue;
                }
                const resource = entry.resource;
                if (resource.resourceType === 'Condition' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-primary-cancer-condition')) {
                    const tempPrimaryCancerCondition = {
                        clinicalStatus: this.lookup(resource, 'clinicalStatus.coding'),
                        meta_profile: 'mcode-primary-cancer-condition',
                        histologyMorphologyBehavior: [],
                        coding: this.lookup(resource, 'code.coding'),
                        id: this.lookup(resource, 'id')[0]
                    };
                    if (this.lookup(resource, 'extension').length !== 0) {
                        this.lookup(resource, 'extension').forEach((_, index) => {
                            if (this.lookup(resource, `extension[${index}].url`)[0].includes('mcode-histology-morphology-behavior')) {
                                tempPrimaryCancerCondition.histologyMorphologyBehavior = this.lookup(resource, `extension[${index}].valueCodeableConcept.coding`);
                            }
                        });
                    }
                    this.primaryCancerCondition.push(tempPrimaryCancerCondition);
                }
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-tnm-clinical-stage-group')) {
                    this.TNMClinicalStageGroup = this.addCoding(this.TNMClinicalStageGroup, this.lookup(resource, 'valueCodeableConcept.coding'));
                }
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-tnm-pathological-stage-group')) {
                    this.TNMPathologicalStageGroup = this.addCoding(this.TNMPathologicalStageGroup, this.lookup(resource, 'valueCodeableConcept.coding'));
                }
                if (resource.resourceType === 'Condition' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-secondary-cancer-condition')) {
                    const tempSecondaryCancerCondition = {
                        clinicalStatus: this.lookup(resource, 'clinicalStatus.coding'),
                        meta_profile: 'mcode-secondary-cancer-condition',
                        id: this.lookup(resource, 'id')[0],
                        bodySite: this.lookup(resource, 'bodySite.coding'),
                        coding: this.lookup(resource, 'code.coding')
                    };
                    this.secondaryCancerCondition.push(tempSecondaryCancerCondition); // needs specific de-dup helper function
                }
                if (resource.resourceType === 'Patient' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-cancer-patient')) {
                    if (this.lookup(resource, 'birthDate').length !== 0) {
                        this.birthDate = this.lookup(resource, 'birthDate')[0];
                    }
                    else {
                        this.birthDate = 'NA';
                    }
                }
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-tumor-marker')) {
                    const tempTumorMarker = {
                        valueQuantity: this.lookup(resource, 'valueQuantity'),
                        valueRatio: this.lookup(resource, 'valueRatio'),
                        valueCodeableConcept: this.lookup(resource, 'valueCodeableConcept.coding'),
                        interpretation: this.lookup(resource, 'interpretation.coding'),
                        coding: this.lookup(resource, 'code.coding')
                    };
                    this.tumorMarker.push(tempTumorMarker);
                }
                // Parse and Extract mCODE Cancer Genetic Variant
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-cancer-genetic-variant')) {
                    const tempCGV = {
                        coding: this.lookup(resource, 'code.coding'),
                        component: {
                            geneStudied: [],
                            genomicsSourceClass: []
                        },
                        valueCodeableConcept: [],
                        interpretation: []
                    };
                    for (const currentComponent of this.lookup(resource, 'component')) {
                        if (currentComponent.code == undefined) {
                            continue;
                        }
                        else {
                            for (const currentComponentCode of currentComponent.code.coding) {
                                if (currentComponentCode.code == '48018-6') {
                                    // With this code, we've reached a GeneStudied. Populate the GeneStudied attribute.
                                    tempCGV.component.geneStudied.push(currentComponent);
                                }
                                if (currentComponentCode.code == '48002-0') {
                                    // With this code, we've reached a GenomicSourceClass. Populate the GenomicSourceClass attribute.
                                    tempCGV.component.genomicsSourceClass.push(currentComponent);
                                }
                            }
                        }
                    }
                    tempCGV.valueCodeableConcept = this.lookup(resource, 'valueCodeableConcept.coding');
                    tempCGV.interpretation = this.lookup(resource, 'interpretation.coding');
                    this.cancerGeneticVariant.push(tempCGV);
                }
                if (resource.resourceType === 'Procedure' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-cancer-related-radiation-procedure')) {
                    const tempCancerRelatedRadiationProcedure = {
                        bodySite: this.lookup(resource, 'bodySite.coding'),
                        mcodeTreatmentIntent: [],
                        coding: this.lookup(resource, 'code.coding')
                    };
                    if (!this.listContainsProcedure(this.cancerRelatedRadiationProcedure, tempCancerRelatedRadiationProcedure)) {
                        this.cancerRelatedRadiationProcedure.push(tempCancerRelatedRadiationProcedure);
                    }
                }
                if (resource.resourceType === 'Procedure' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-cancer-related-surgical-procedure')) {
                    const tempCancerRelatedSurgicalProcedure = {
                        bodySite: this.lookup(resource, 'bodySite.coding'),
                        reasonReference: this.lookup(resource, 'reasonReference')[0],
                        coding: this.lookup(resource, 'code.coding')
                    };
                    if (!this.listContainsProcedure(this.cancerRelatedSurgicalProcedure, tempCancerRelatedSurgicalProcedure)) {
                        this.cancerRelatedSurgicalProcedure.push(tempCancerRelatedSurgicalProcedure);
                    }
                }
                if (resource.resourceType === 'MedicationStatement' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-cancer-related-medication-statement')) {
                    this.cancerRelatedMedicationStatement = this.addCoding(this.cancerRelatedMedicationStatement, this.lookup(resource, 'medicationCodeableConcept.coding'));
                }
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-ecog-performance-status')) {
                    this.ecogPerformanceStatus = this.lookup(resource, 'valueInteger')[0]; // this is probably bad type handling
                }
                if (resource.resourceType === 'Observation' &&
                    this.resourceProfile(this.lookup(resource, 'meta.profile'), 'mcode-karnofsky-performance-status')) {
                    this.karnofskyPerformanceStatus = this.lookup(resource, 'valueInteger')[0]; // so is this
                }
            }
        }
        else {
            throw Error("Input Patient Bundle is null.");
        }
        // Checking if the performanceStatus exists and also making sure it's not 0, as 0 is a valid score
        if (!this.ecogPerformanceStatus && this.ecogPerformanceStatus != 0) {
            this.ecogPerformanceStatus = undefined;
        }
        if (!this.karnofskyPerformanceStatus && this.karnofskyPerformanceStatus != 0) {
            this.karnofskyPerformanceStatus = undefined;
        }
        // Once all resources are loaded, check to add the meta.profile for cancer related surgical procedure reason references.
        for (const procedure of this.cancerRelatedSurgicalProcedure) {
            const conditions = this.primaryCancerCondition.concat(this.secondaryCancerCondition);
            const reasonReference = procedure.reasonReference;
            for (const condition of conditions) {
                if (condition.id === reasonReference.reference) {
                    const reasonReferenceResult = {
                        reference: reasonReference.reference,
                        display: reasonReference.display,
                        meta_profile: condition.meta_profile
                    };
                    procedure.reasonReference = reasonReferenceResult;
                }
            }
        }
    }
    /**
     * Getters.
     */
    getPrimaryCancerConditions() {
        return this.primaryCancerCondition;
    }
    getSecondaryCancerConditions() {
        return this.secondaryCancerCondition;
    }
    getTNMclinicalStageGroup() {
        return this.TNMClinicalStageGroup;
    }
    getTNMpathologicalStageGroup() {
        return this.TNMPathologicalStageGroup;
    }
    getBirthDate() {
        return this.birthDate;
    }
    getTumorMarkers() {
        return this.tumorMarker;
    }
    getCancerGeneticVariants() {
        return this.cancerGeneticVariant;
    }
    getCancerRelatedRadiationProcedures() {
        return this.cancerRelatedRadiationProcedure;
    }
    getCancerRelatedSurgicalProcedures() {
        return this.cancerRelatedSurgicalProcedure;
    }
    getCancerRelatedMedicationStatements() {
        return this.cancerRelatedMedicationStatement;
    }
    getEcogPerformanceStatus() {
        return this.ecogPerformanceStatus ? this.ecogPerformanceStatus : -1;
    }
    getKarnofskyPerformanceStatus() {
        return this.karnofskyPerformanceStatus ? this.karnofskyPerformanceStatus : -1;
    }
    /**
     * Resource lookup.
     * @param resource
     * @param path
     * @param environment
     * @returns
     */
    lookup(resource, path, environment) {
        // The FHIR client Resource type definition is wrong (sort of) - it requires
        // that Meta references have a lastUpdated time. This is, in fact, optional,
        // so just jam it in.
        return fhirpath_1.default.evaluate(resource, path, environment);
    }
    /**
     * Checks if the given profile constains a match to the given key.
     */
    resourceProfile(profiles, key) {
        for (const profile of profiles) {
            if (profile.includes(key)) {
                return true;
            }
        }
        return false;
    }
    /**
     * Checks whether the given list contains the given coding.
     * @param codingList
     * @param coding
     * @returns
     */
    contains(codingList, coding) {
        return codingList.some((list_coding) => list_coding.system === coding.system && list_coding.code === coding.code);
    }
    /**
     * Adds the given coding to the given coding list.
     * @param codingList
     * @param codes
     * @returns
     */
    addCoding(codingList, codes) {
        for (const code of codes) {
            if (!this.contains(codingList, code)) {
                codingList.push(code);
            }
        }
        return codingList;
    }
    /**
     * Returns whether the given list contains the given procedure.
     * @param procedureList
     * @param procedure
     * @returns
     */
    listContainsProcedure(procedureList, procedure) {
        for (const storedProcedure of procedureList) {
            if (procedure.coding.every((coding1) => storedProcedure.coding.some((coding2) => coding1.system == coding2.system && coding1.code == coding2.code)) &&
                (!procedure.bodySite ||
                    !storedProcedure.bodySite ||
                    procedure.bodySite.every((coding1) => storedProcedure.coding.some((coding2) => coding1.system == coding2.system && coding1.code == coding2.code)))) {
                return true;
            }
        }
        return false;
    }
}
exports.mCODEextractor = mCODEextractor;
