import { CodeableConcept, ContactDetail, FhirResource, Identifier, Location, Reference, ResearchStudy as IResearchStudy, ResearchStudyArm, ResearchStudyObjective, Resource } from 'fhir/r4';
/**
 * Utility function to convert a list of strings into a list of CodeableConcepts.
 * If given a string that "looks like" it's a JSON array (starts and ends with '[' ']') this will attempt to parse the
 * list as JSON. (If the JSON is invalid, the exception from JSON.parse will be propegated.) Otherwise, if the
 * conditions is a string, it will be split on commas.
 *
 * @param conditions a list of strings to convert to CodeableConcepts
 */
export declare function convertStringsToCodeableConcept(conditions: string | string[]): CodeableConcept[];
export declare type Container<TContainer, TContained, K extends keyof TContainer> = {
    [P in K]: Array<TContained> | undefined;
};
export declare function addToContainer<TContainer, TContained, K extends keyof TContainer>(container: Container<TContainer, TContained, K>, propertyName: K, item: TContained): void;
export declare function addContainedResource(researchStudy: IResearchStudy, resource: FhirResource): Reference;
/**
 * Looks up a contained resource by ID. This works by scanning through every contained resource in the research study,
 * so it ultimately operates in O(n) time! If you need to continually look up references, it's recommended that to
 * instead create a Map<string, ContainedResource> to look them up.
 * @param id the ID of the resource to look up
 */
export declare function getContainedResource(researchStudy: IResearchStudy, id: string): FhirResource | null;
/**
 * Creates a Reference to a resource, assuming it will be a contained resource.
 * The resource must have an `id` set on it, otherwise this will raise an error.
 * @param resource the resource to create a Reference to
 */
export declare function createReferenceTo(resource: Resource): Reference;
/**
 * A basic ResearchStudy implementation, this provides helper methods for
 * doing things like adding contact information.
 */
export declare class ResearchStudy implements IResearchStudy {
    resourceType: "ResearchStudy";
    id?: string;
    identifier?: Identifier[];
    title?: string;
    protocol?: Reference[];
    status: IResearchStudy['status'];
    phase?: CodeableConcept;
    category?: CodeableConcept[];
    condition?: CodeableConcept[];
    contact?: ContactDetail[];
    keyword?: CodeableConcept[];
    location?: CodeableConcept[];
    description?: string;
    arm?: ResearchStudyArm[];
    objective?: ResearchStudyObjective[];
    enrollment?: Reference[];
    sponsor?: Reference;
    principalInvestigator?: Reference;
    site?: Reference[];
    contained?: FhirResource[];
    constructor(id: string | number);
    /**
     * Add a contained resource, returning a reference to it that can be added
     * elsewhere. If the contained resource does not have an ID, one will be
     * created, based on the resource type.
     * @param resource the resource to add
     * @return a reference to the contained resource
     */
    addContainedResource(resource: FhirResource): Reference;
    /**
     * Creates a new, probably unique ID for a contained resource. (At present
     * this doesn't go through the contained resources to ensure the ID is
     * actually unique.)
     * @param prefix the prefix for the reference
     */
    createReferenceId: (prefix?: string) => string;
    /**
     * Adds a contact to the contact field.
     *
     * @param contact the contact to add
     */
    addContact(contact: ContactDetail): ContactDetail;
    /**
     * Adds a contact to the contact field.
     *
     * @param name the name of the contact
     * @param phone the work phone number of the contact
     * @param email the work email of the contact
     * @returns the newly created contact
     */
    addContact(name: string, phone?: string, email?: string): ContactDetail;
    /**
     * Adds a site as a contained resource.
     * @param name the name of the site to add
     * @return the location added
     */
    addSite(name: string, phone?: string, email?: string): Location;
    addSite(location: Location): Location;
}
export default ResearchStudy;
