"use strict";
// FHIR data types supporting ResearchStudy
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResearchStudy = exports.createReferenceTo = exports.getContainedResource = exports.addContainedResource = exports.addToContainer = exports.convertStringsToCodeableConcept = void 0;
/**
 * Utility function to convert a list of strings into a list of CodeableConcepts.
 * If given a string that "looks like" it's a JSON array (starts and ends with '[' ']') this will attempt to parse the
 * list as JSON. (If the JSON is invalid, the exception from JSON.parse will be propegated.) Otherwise, if the
 * conditions is a string, it will be split on commas.
 *
 * @param conditions a list of strings to convert to CodeableConcepts
 */
function convertStringsToCodeableConcept(conditions) {
    if (typeof conditions === 'string') {
        if (conditions.startsWith('[') && conditions.endsWith(']')) {
            conditions = JSON.parse(conditions);
        }
    }
    if (!Array.isArray(conditions)) {
        conditions = conditions.split(/\s*,\s*/);
    }
    const jsonConditions = conditions;
    const fhirConditions = [];
    for (const condition of jsonConditions) {
        fhirConditions.push({ text: condition });
    }
    return fhirConditions;
}
exports.convertStringsToCodeableConcept = convertStringsToCodeableConcept;
function addToContainer(container, propertyName, item) {
    if (!container[propertyName]) {
        container[propertyName] = [];
    }
    // I'm unclear why TypeScript won't make this inference when the type is
    // optional, but it won't
    container[propertyName].push(item);
}
exports.addToContainer = addToContainer;
function addContainedResource(researchStudy, resource) {
    if (!researchStudy.contained)
        researchStudy.contained = [];
    researchStudy.contained.push(resource);
    return createReferenceTo(resource);
}
exports.addContainedResource = addContainedResource;
/**
 * Looks up a contained resource by ID. This works by scanning through every contained resource in the research study,
 * so it ultimately operates in O(n) time! If you need to continually look up references, it's recommended that to
 * instead create a Map<string, ContainedResource> to look them up.
 * @param id the ID of the resource to look up
 */
function getContainedResource(researchStudy, id) {
    if (researchStudy.contained) {
        // Lookup has to be done sequentially for now
        for (const contained of researchStudy.contained) {
            if (contained.id === id) {
                return contained;
            }
        }
    }
    // Contained empty/nothing found? Return null.
    return null;
}
exports.getContainedResource = getContainedResource;
/**
 * Creates a Reference to a resource, assuming it will be a contained resource.
 * The resource must have an `id` set on it, otherwise this will raise an error.
 * @param resource the resource to create a Reference to
 */
function createReferenceTo(resource) {
    const reference = {};
    if (resource.id) {
        reference.reference = '#' + resource.id;
    }
    else {
        let message = 'no ID to create reference';
        if (resource.resourceType) {
            message += ' to ' + resource.resourceType;
        }
        throw new Error(message);
    }
    if (resource.resourceType) {
        reference.type = resource.resourceType;
    }
    return reference;
}
exports.createReferenceTo = createReferenceTo;
/**
 * A basic ResearchStudy implementation, this provides helper methods for
 * doing things like adding contact information.
 */
class ResearchStudy {
    constructor(id) {
        this.resourceType = 'ResearchStudy';
        this.status = 'active';
        if (typeof id === 'number') {
            // This is mostly for convenience of using array indices as IDs
            this.id = 'study-' + id;
        }
        else {
            // Pointless toString is to support use from non-TypeScript apps that
            // try and use something that isn't a string.
            this.id = id.toString();
        }
        // This is done as a closure to avoid adding an enumerable property that
        // would show up in JSON output
        this.createReferenceId = (function () {
            let generatedId = 0;
            return function (prefix = 'resource') {
                return prefix + '-' + (generatedId++);
            };
        })();
    }
    /**
     * Add a contained resource, returning a reference to it that can be added
     * elsewhere. If the contained resource does not have an ID, one will be
     * created, based on the resource type.
     * @param resource the resource to add
     * @return a reference to the contained resource
     */
    addContainedResource(resource) {
        if (!this.contained)
            this.contained = [];
        this.contained.push(resource);
        if (!resource.id) {
            resource.id = this.createReferenceId(resource.resourceType);
        }
        return createReferenceTo(resource);
    }
    addContact(nameOrContact, phone, email) {
        const contact = typeof nameOrContact === 'string' ? {} : nameOrContact;
        if (typeof nameOrContact === 'string') {
            contact.name = nameOrContact;
            if (phone || email) {
                const telecoms = [];
                if (phone) {
                    telecoms.push({ system: 'phone', value: phone, use: 'work' });
                }
                if (email) {
                    telecoms.push({ system: 'email', value: email, use: 'work' });
                }
                contact.telecom = telecoms;
            }
        }
        if (!this.contact)
            this.contact = [];
        this.contact.push(contact);
        return contact;
    }
    addSite(nameOrLocation, phone, email) {
        const location = typeof nameOrLocation === 'string' ?
            { resourceType: 'Location', id: this.createReferenceId('location'), name: nameOrLocation } :
            nameOrLocation;
        if (typeof nameOrLocation === 'string') {
            // Also possibly add the telecoms
            if (phone) {
                addToContainer(location, 'telecom', { system: 'phone', value: phone, use: 'work' });
            }
            if (email) {
                addToContainer(location, 'telecom', { system: 'email', value: email, use: 'work' });
            }
        }
        addToContainer(this, 'site', this.addContainedResource(location));
        return location;
    }
}
exports.ResearchStudy = ResearchStudy;
exports.default = ResearchStudy;
