import { BundleEntry, BundleEntrySearch, ResearchStudy } from 'fhir/r4';
import { SearchSet as SearchSetBundle } from './server';
/**
 * Same as a BundleEntry but with the search parameters required.
 */
export interface SearchBundleEntry extends BundleEntry<ResearchStudy> {
    search: BundleEntrySearch;
}
export declare type SearchEntryMode = Exclude<BundleEntrySearch['mode'], undefined>;
export declare class SearchSet implements SearchSetBundle {
    resourceType: "Bundle";
    type: "searchset";
    /**
     * The total number of results. As a bundle support pagination, this defaults
     * to the current number of studies, but may be overridden. (See
     * https://www.hl7.org/fhir/http.html#paging for details on paging in FHIR.)
     * Note that addEntry will increment the total by one on each call.
     */
    total: number;
    entry: SearchBundleEntry[];
    /**
     * Creates a new SearchSet, adding the given array of studies with a default
     * score of 1.0.
     * @param entries the entries to add
     */
    constructor(entries?: Array<ResearchStudy | SearchBundleEntry>);
    addEntries(...entries: Array<ResearchStudy | SearchBundleEntry>): void;
    /**
     * Add a study to the searchset with no defined score and the search mode set
     * to "match".
     *
     * @param study the study to add
     */
    addEntry(entry: SearchBundleEntry): void;
    /**
     * Add a study to the searchset with no score and the given search mode
     *
     * @param study the study to add
     * @param mode the mode
     */
    addEntry(study: ResearchStudy, mode: SearchEntryMode): void;
    /**
     * Add a study to the searchset.
     *
     * If the score is not given, then the score will be left unset in the entry.
     * Otherwise, if the score will be clamped to the range [0..1] (values less
     * than 0 become 0, values greater than 1 become 1). NaN is treated as
     * "undefined" and leaves the score unset.
     *
     * @param study the study to add
     * @param score the score, from [0..1]
     * @param mode the mode, defaults to 'match'
     */
    addEntry(study: ResearchStudy, score?: number, mode?: SearchEntryMode): void;
    addEntry(studyOrEntry: SearchBundleEntry | ResearchStudy): void;
}
export default SearchSet;
