"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchSet = void 0;
const fhir_type_guards_1 = require("./fhir-type-guards");
class SearchSet {
    /**
     * Creates a new SearchSet, adding the given array of studies with a default
     * score of 1.0.
     * @param entries the entries to add
     */
    constructor(entries) {
        this.resourceType = 'Bundle';
        // Class attributes
        this.type = 'searchset';
        /**
         * The total number of results. As a bundle support pagination, this defaults
         * to the current number of studies, but may be overridden. (See
         * https://www.hl7.org/fhir/http.html#paging for details on paging in FHIR.)
         * Note that addEntry will increment the total by one on each call.
         */
        this.total = 0;
        this.entry = [];
        if (entries) {
            this.addEntries(...entries);
        }
    }
    addEntries(...entries) {
        for (const entry of entries) {
            this.addEntry(entry);
        }
    }
    addEntry(studyOrEntry, scoreOrMode = null, mode) {
        let score = null;
        if (typeof scoreOrMode === 'string') {
            // There is no overload that allows two modes so if anyone did that they
            // did it outside of TypeScript so go ahead and ignore them
            mode = scoreOrMode;
        }
        else if (typeof scoreOrMode === 'number') {
            score = scoreOrMode;
        }
        const entry = (0, fhir_type_guards_1.isResearchStudy)(studyOrEntry)
            ? { resource: studyOrEntry, search: { mode: mode !== null && mode !== void 0 ? mode : 'match' } }
            : studyOrEntry;
        // Grab the score out of the entry if one was given
        // (See above on how there is no overload for both giving an entry and a
        // score - if you do that, you're ignoring TypeScript anyway)
        if (entry.search.score)
            score = entry.search.score;
        if (score !== null && !Number.isNaN(score)) {
            // Clamp the range of the score from 0 to 1
            entry.search.score = Math.min(1, Math.max(score, 0));
        }
        this.entry.push(entry);
        this.total++;
    }
}
exports.SearchSet = SearchSet;
exports.default = SearchSet;
