/// <reference types="node" />
import express from 'express';
import { Bundle, ResearchStudy } from 'fhir/r4';
import * as http from 'http';
export interface SearchSet<T = ResearchStudy> extends Bundle<T> {
    type: 'searchset';
}
export declare type ClinicalTrialMatcher = (patientBundle: Bundle) => Promise<SearchSet>;
/**
 * Server configuration.
 */
export interface Configuration {
    [key: string]: string | number | undefined;
    /**
     * The port to listen on (must be in the range 0-65535, if outside or unset,
     * the default of 3000 is used).
     */
    port?: number;
    /**
     * The host address to bind to, if unset, uses the default from
     * http.Server#listen.
     */
    host?: string;
    /**
     * If defined, a prefix which will be prepended to each mapped URL. If this
     * is not specified by the PASSENGER_BASE_URI is defined in the process
     * environment, then PASSENGER_BASE_URI will be used as the prefix. (To
     * prevent that, you can set the prefix explicitly to '/' which will prevent
     * any prefix from being used.)
     */
    urlPrefix?: string;
}
/**
 * Server options - configuration options that are more involved that simple
 * strings.
 */
export interface ServerOptions {
    /**
     * If given, use the given Express engine rather than creating a new one.
     */
    appEngine?: express.Application;
}
/**
 * The JSON request from the client. This exists more for documentation purposes
 * than anything else.
 */
export interface ClinicalTrialMatchServiceRequest {
    /**
     * The patient data. If a string, it's parsed as JSON. The string use should
     * be deprecated.
     */
    patientData: Bundle | string;
}
export declare class ClinicalTrialMatchingService {
    matcher: ClinicalTrialMatcher;
    readonly app: express.Application;
    private readonly configuration;
    private _server;
    /**
     * Create a new service.
     * @param matcher the matcher function
     * @param configuration the server configuration
     */
    constructor(matcher: ClinicalTrialMatcher);
    constructor(matcher: ClinicalTrialMatcher, options: ServerOptions);
    constructor(matcher: ClinicalTrialMatcher, configuration: Configuration, options?: ServerOptions);
    /**
     * Gets the server object, if it's running, or `null` if it isn't.
     */
    get server(): http.Server | null;
    /**
     * Gets the configured port. If the port configuration is invalid, this
     * returns the default port, 3000.
     */
    get port(): number;
    get host(): string | undefined;
    getClinicalTrial(request: express.Request, response: express.Response): void;
    /**
     * Closes the server if it's running. Unlike the underlying service function,
     * no error is raised if the server wasn't running.
     */
    close(): Promise<void>;
    /**
     * Logs a message. The default simply invokes console.log, but subclasses
     * may override to use more fine-grained logging.
     * @param message the message to log
     */
    log(message: string): void;
    /**
     * Starts the server running, returning the newly running instance. If the
     * server is already running, simply returns the already running instance.
     */
    listen(): Promise<http.Server>;
}
export default ClinicalTrialMatchingService;
