"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClinicalTrialMatchingService = void 0;
const express_1 = __importDefault(require("express"));
const body_parser_1 = __importDefault(require("body-parser"));
const errors_1 = require("./errors");
const fhir_type_guards_1 = require("./fhir-type-guards");
function isConfiguration(o) {
    if (typeof o !== 'object' || o === null)
        return false;
    // For now, an object is a configuration object if every key is a string or number
    const obj = o;
    for (const k in obj) {
        const t = typeof obj[k];
        if (t !== 'string' && t !== 'number' && t !== 'undefined')
            return false;
    }
    return true;
}
class ClinicalTrialMatchingService {
    constructor(matcher, configurationOrOptions, options) {
        this.matcher = matcher;
        this._server = null;
        if (isConfiguration(configurationOrOptions)) {
            this.configuration = configurationOrOptions;
        }
        else {
            // If here, no configuration was given so use whatever options we have
            options = configurationOrOptions;
            this.configuration = {};
        }
        // if called with only options, options will have been moved to the options variable
        this.app = options && options.appEngine ? options.appEngine : (0, express_1.default)();
        this.app.use(body_parser_1.default.json({
            // Need to increase the payload limit to receive patient bundles
            limit: '10MB'
        }));
        // Set up CORS
        this.app.use(function (_req, res, next) {
            // Website you wish to allow to connect
            res.setHeader('Access-Control-Allow-Origin', '*');
            // Request methods you wish to allow
            res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');
            // Request headers you wish to allow
            res.setHeader('Access-Control-Allow-Headers', 'Cache-Control, Pragma, Origin, Authorization, Content-Type, X-Requested-With, WU-Api-Key, WU-Api-Secret');
            next();
        });
        let prefix;
        if (this.configuration.urlPrefix) {
            prefix = this.configuration.urlPrefix;
        }
        else if (process.env['PASSENGER_BASE_URI']) {
            prefix = process.env['PASSENGER_BASE_URI'];
        }
        else {
            prefix = '/';
        }
        // Make sure the prefix is of the form /path without a trailing slash and
        // with a leading slash
        if (!prefix.startsWith('/')) {
            prefix = '/' + prefix;
        }
        // Remove the trailing slash - note that this means a prefix of '/' becomes
        // the empty string, WHICH IS EXPECTED
        if (prefix.endsWith('/')) {
            prefix = prefix.substr(0, prefix.length - 1);
        }
        // Default callback
        this.app.get(prefix.length > 0 ? prefix : '/', (_req, res) => {
            res.status(200).send('Hello from the Clinical Trial Matching Service');
        });
        this.app.post(prefix + '/getClinicalTrial', (request, response) => {
            this.getClinicalTrial(request, response);
        });
    }
    /**
     * Gets the server object, if it's running, or `null` if it isn't.
     */
    get server() {
        return this._server;
    }
    /**
     * Gets the configured port. If the port configuration is invalid, this
     * returns the default port, 3000.
     */
    get port() {
        if (typeof this.configuration.port === 'number') {
            const port = Math.floor(this.configuration.port);
            if (port >= 0 && port <= 0xffff)
                return port;
        }
        return 3000;
    }
    get host() {
        return typeof this.configuration.host === 'string' ? this.configuration.host : undefined;
    }
    getClinicalTrial(request, response) {
        const patientBundle = request.body;
        if ((0, fhir_type_guards_1.isBundle)(patientBundle)) {
            // Error handler for exceptions raised (as it should be handled on the
            // resulting Promise and if invoking the matcher itself fails)
            const handleError = (error) => {
                if ((0, errors_1.isHttpError)(error)) {
                    response.status((0, errors_1.restrictToHttpErrors)(error.httpStatus)).send({ error: error.message });
                }
                else {
                    console.error('An unexpected internal server error occurred:');
                    console.error(error);
                    response
                        .status(500)
                        .send({ error: 'Internal server error', exception: Object.prototype.toString.call(error) });
                }
            };
            try {
                this.matcher(patientBundle)
                    .then((result) => {
                    response.status(200).send(JSON.stringify(result));
                })
                    .catch(handleError);
            }
            catch (ex) {
                handleError(ex);
            }
        }
        else {
            response.status(400).send({ error: 'Invalid patientBundle' });
        }
    }
    /**
     * Closes the server if it's running. Unlike the underlying service function,
     * no error is raised if the server wasn't running.
     */
    close() {
        if (this._server === null) {
            return Promise.resolve();
        }
        else {
            const server = this._server;
            return new Promise((resolve, reject) => {
                server.close((err) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
                this._server = null;
            });
        }
    }
    /**
     * Logs a message. The default simply invokes console.log, but subclasses
     * may override to use more fine-grained logging.
     * @param message the message to log
     */
    log(message) {
        console.log(message);
    }
    /**
     * Starts the server running, returning the newly running instance. If the
     * server is already running, simply returns the already running instance.
     */
    listen() {
        if (this._server !== null) {
            return Promise.resolve(this._server);
        }
        return new Promise((resolve, reject) => {
            const port = this.port;
            const host = this.host;
            // It's unclear if we can pass undefined to listen
            const server = host ? this.app.listen(port, host) : this.app.listen(port);
            this._server = server;
            server.once('error', (error) => {
                reject(error);
            });
            server.once('listening', () => {
                const listeningOn = server.address();
                if (typeof listeningOn === 'object' && listeningOn !== null) {
                    this.log(`Server listening on ${listeningOn.address}:${listeningOn.port}...`);
                }
                else {
                    // Should not be possible at this point but whatever
                    this.log(`Server listening on ${listeningOn === null ? 'unknown address' : listeningOn}.`);
                }
                resolve(server);
            });
        });
    }
}
exports.ClinicalTrialMatchingService = ClinicalTrialMatchingService;
exports.default = ClinicalTrialMatchingService;
